function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react'; //eslint-disable-line no-unused-vars

import deepEqual from 'deep-eql';
import isPromise from 'is-promise';
var STATUS_FETCHING = 0;
var STATUS_LOADED = 1;
var STATUS_ERROR = -1;
var NODE_ENV = undefined || 'production';
var debug = NODE_ENV === 'development' ? console.log : () => {};
debug('Environment:', NODE_ENV);
export default function ReactLoader() {
  var _options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var options = Object.assign({
    load: null,
    resultProp: 'data',
    errorProp: 'error',
    errorComponent: () => /*#__PURE__*/React.createElement("div", null, "Impossible to fetch the data requested."),
    loadingComponent: () => /*#__PURE__*/React.createElement("div", null, "Loading..."),
    shouldComponentReload: (props, nextProps) => !deepEqual(props, nextProps),
    componentWillUnmount: () => {}
  }, _options);

  if (!options.load) {
    throw new Error('ReactLoader : No load() defined. Cannot create');
  }

  if (typeof options.load !== 'function') {
    throw new Error('ReactLoader : load must be a function returning a Promise. Cannot create');
  }

  var ErrorComponent = options.errorComponent;
  var LoadingComponent = options.loadingComponent;
  var load = options.load;
  var componentWillUnmount = options.componentWillUnmount;
  var shouldComponentReload = options.shouldComponentReload;
  var resultProp = options.resultProp;
  var errorProp = options.errorProp;

  class _Loader extends React.Component {
    constructor(props) {
      super(props);
      this.state = {
        status: STATUS_FETCHING
      };
    }

    componentDidMount() {
      this.doLoad();
    } //eslint-disable-next-line no-unused-vars


    componentDidUpdate(prevProps, prevState, snapshot) {
      if (shouldComponentReload(prevProps, this.props)) {
        this.doLoad();
      }
    }

    doLoad() {
      debug('doLoad', this.props);
      this.setState({
        status: STATUS_FETCHING
      });
      var prom = load(this.props);

      if (!isPromise(prom)) {
        throw new TypeError('ReactLoader : load(props) must return a Promise/A+ compliant object');
      }

      prom.then(data => {
        debug('Loaded !', this.props);
        this.setState({
          status: STATUS_LOADED,
          data: data
        });
        return data;
      }).catch(e => {
        debug('Error !', this.props);
        this.setState({
          status: STATUS_ERROR,
          error: e
        });
      });
    }

    componentWillUnmount() {
      debug('unmount !', this.props);
      componentWillUnmount(this.props);
    }

    render() {
      debug('render');
      var props = this.props;

      if (this.state.status === STATUS_ERROR) {
        var forwardProps = Object.assign({}, props);
        forwardProps[errorProp] = this.state.error;
        return /*#__PURE__*/React.createElement(ErrorComponent, forwardProps);
      } else if (this.state.status === STATUS_LOADED) {
        return this.renderComponent();
      } else {
        return /*#__PURE__*/React.createElement(LoadingComponent, props);
      }
    } //eslint-disable-next-line no-unused-vars


    shouldComponentUpdate(nextProps, nextState, nextContext) {
      return !deepEqual(this.props, nextProps) || !deepEqual(this.state, nextState);
    }

  }

  var outFn = Component => {
    if (typeof Component === 'object' && Component.kind === 'class') {
      return _objectSpread(_objectSpread({}, Component), {}, {
        finisher: Class => function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          return new (outFn(Class))(...args);
        }
      });
    } else {
      class Loader extends _Loader {
        renderComponent() {
          var forwardProps = Object.assign({}, this.props);
          forwardProps[resultProp] = this.state.data;
          return /*#__PURE__*/React.createElement(Component, forwardProps);
        }

      }

      Loader.displayName = "ReactLoader(".concat(Component.displayName || Component.name, ")");
      Loader.propTypes = Component.propTypes;
      return Loader;
    }
  };

  return outFn;
}